function fill_ice_velocity(migration_distance)

% add paths to libraries
if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
    addpath(genpath('/resstore/b0133/Data/Velocity'));
end

%% Stop warnings 
warning('off','all')

%% Load flux gate
g = load('../out/pub/gates/flux_gates_migrated.mat');
migration_idx = find(g.migration_distance == migration_distance); % metres
gate_x = g.gate_x_migrate{migration_idx};
gate_y = g.gate_y_migrate{migration_idx};
gate_nan = isnan(gate_x); % indices of gate nans
clear g

%% Load raw velocity
load(['../out/int/gate_data/gate_vel_raw_linear_migrate_' num2str(migration_distance) 'm_compiled.mat'],'sp_raw','spE_raw','source_vel','time_vel')

%% Load reference velocity
ref = load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm_fast.mat'],'gate_vx','gate_vy','gate_vx_err','gate_vy_err','gate_fd','gate_theta');
ref.sp = hypot(ref.gate_vx,ref.gate_vy);
ref.ctv = abs(ref.gate_vx.*sin(ref.gate_theta) - ref.gate_vy.*cos(ref.gate_theta));

%% Freeze slow-flowing pixels
slow = ref.sp<=50;
sp_raw(slow,:) = repmat(ref.sp(slow),[1,length(time_vel)]);

%% Early S1 data causes problems, so set them to NaN
remove = time_vel>=datenum(2015,7,2) & time_vel<=datenum(2015,11,16);
sp_raw(:,remove) = NaN;
spE_raw(:,remove) = NaN;
clear remove

%% Do an initial sift for outliers which can skew the alignment
remove = sp_raw>=ref.sp.*2 | sp_raw<=ref.sp.*0.5;
sp_raw(remove) = NaN;
spE_raw(remove) = NaN;

%% Align datasets

% Align datasources and remove some layers that turned out to be unhelpful
% (e.g. multi-year coarse velocities in the middle of monthly high-res
% velocities) 
alignments = struct();
[sp_raw,alignments.I_vs_M] = align_measures_with_itslive_fit(sp_raw,source_vel,time_vel,slow);
[sp_raw,alignments.I_vs_M] = align_measures_with_itslive_fit2(sp_raw,source_vel,time_vel,slow);
[sp_raw,alignments.I_vs_MY] = align_multiyear_with_itslive(sp_raw,source_vel,time_vel);
[sp_raw,alignments.M_vs_MY] = align_multiyear_with_measures(sp_raw,source_vel,time_vel);
[sp_raw,alignments.E_vs_ann] = align_annual_with_enveo(sp_raw,source_vel,time_vel);
[sp_raw,alignments.J_vs_ann] = align_annual_with_joughin(sp_raw,source_vel,time_vel);
[sp_raw,alignments.Isim_vs_ann] = align_annual_with_Isim(sp_raw,source_vel,time_vel);
[sp_raw,alignments.E_vs_SID] = align_SID_with_enveo(sp_raw,source_vel,time_vel,single(slow),gate_x,gate_y);
[sp_raw,alignments.SID_vs_ann] = align_SID_with_annual_fit(sp_raw,source_vel,time_vel,single(slow));

%% Remove some unwanted layers from the stack
remove = zeros([1,length(source_vel)]);
remove(source_vel==4 & time_vel<=datenum(1996,1,1)) = 1; % remove ITSLIVE before 1996
%remove(source_vel==4 & time_vel>=datenum(2006,1,1)) = 1; % remove ITSLIVE after 2006
%remove(source_vel==1 & time_vel>=datenum(2018,1,1)) = 1; % remove MEaSUREs annual after 2018
remove(source_vel==5 & time_vel>=datenum(2015,1,1)) = 1; % remove MEaSUREs multiyear mosaic after 2015 (once we have monthly)
remove(source_vel==2) = 1; % remove all ENVEO data
remove = logical(remove);
sp_raw(:,remove) = [];
spE_raw(:,remove) = [];
source_vel(remove) = [];
time_vel(remove) = [];
clear remove

%% Calculate vx and vy from speed and flow direction
vx_raw = single(zeros(size(sp_raw))).*NaN;
vy_raw = single(zeros(size(sp_raw))).*NaN;
vxE_raw = single(zeros(size(sp_raw))).*NaN;
vyE_raw = single(zeros(size(sp_raw))).*NaN;
isf = isfinite(sp_raw);
fd_all = repmat(ref.gate_fd,[1,size(sp_raw,2)]);
x_all = repmat(gate_x,[1,size(sp_raw,2)]);
y_all = repmat(gate_y,[1,size(sp_raw,2)]);
[vx_raw(isf),vy_raw(isf)] = compass2cart(fd_all(isf),sp_raw(isf));
[vx_raw(isf),vy_raw(isf)] = uv2vxvy(x_all(isf),y_all(isf),vx_raw(isf),vy_raw(isf));
[vxE_raw(isf),vyE_raw(isf)] = compass2cart(fd_all(isf),spE_raw(isf));
[vxE_raw(isf),vyE_raw(isf)] = uv2vxvy(x_all(isf),y_all(isf),vxE_raw(isf),vyE_raw(isf));
clear fd_all x_all y_all isf spE_raw isf

%% Remove outliers
% initialise
vx_clean = vx_raw;
vy_clean = vy_raw;
vxE_clean = vxE_raw;
vyE_clean = vyE_raw;

% scaling factor for outlier removal 
c=-1/(sqrt(2)*erfcinv(3/2));

% add some negligible noise to the time vector, to permit detrending with
% duplicate times
time_vel = time_vel(:)';
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time_vel));
tmp = time_vel + r;
tmp = sort(tmp);

% Prepare data cleaning
gate_theta = repmat(ref.gate_theta,[1,length(time_vel)]);
sp_raw = abs(vx_clean.*sin(gate_theta) - vy_clean.*cos(gate_theta)); % use the cross-gate velocity to find outliers
removed = zeros(size(vx_clean));
clear gate_theta

% Global outlier removal
%   - stage 1
insuff = sum(isfinite(sp_raw),2) < size(sp_raw,2)*0.3; % need more than 30% of timeseries for outlier removal to be meaningful
sp_detrend = detrend(sp_raw',1,'omitnan','samplepoints',datetime(tmp,'convertfrom','datenum'))'; % remove linear trend from all timeseries; takes a while...
sMAD = c.*nanmedian(abs(sp_detrend-nanmedian(sp_detrend,2)),2);
removed(abs(sp_detrend-nanmedian(sp_detrend,2)) > sMAD.*5) = 1;
removed(insuff | slow,:) = 0; % only apply global filter to timeseries with enough data points
sp_raw(logical(removed)) = NaN;
%   - stage 2
insuff = sum(isfinite(sp_raw),2) < size(sp_raw,2)*0.3; % need more than 30% of timeseries for outlier removal to be meaningful
sp_detrend = detrend(sp_raw',1,'omitnan','samplepoints',datetime(tmp,'convertfrom','datenum'))'; % remove linear trend from all timeseries; takes a while...
sMAD = c.*nanmedian(abs(sp_detrend-nanmedian(sp_detrend,2)),2);
removed(abs(sp_detrend-nanmedian(sp_detrend,2)) > sMAD.*3) = 1;
removed(insuff | slow,:) = 0; % only apply global filter to timeseries with enough data points
sp_raw(logical(removed)) = NaN;

% Local outlier removal
%   - add negligible random noise to time so outlier detection works
time_vel = time_vel(:)';
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time_vel));
tmp = time_vel + r;
tmp = sort(tmp);
%   - stage 1 (4 month window)
tf = isoutlier(sp_raw,'movmedian',125,2,'samplepoints',tmp,'ThresholdFactor', 2); % outliers are those more than 3-scaled MADs from the median in a 90-day moving window
%too_early = time_vel<datenum(2015,1,1); % only use local outlier removal after 2015
%tf(:,too_early) = false; % only use local outlier removal after 2015
removed(tf) = 1; % apply local outlier
sp_raw(logical(removed)) = NaN; 
%   - stage 2 (3 month window)
tf = isoutlier(sp_raw,'movmedian',93,2,'samplepoints',tmp,'ThresholdFactor', 2); % outliers are those more than 3-scaled MADs from the median in a 90-day moving window
%too_early = time_vel<datenum(2015,1,1); % only use local outlier removal after 2015
%tf(:,too_early) = false; % only use local outlier removal after 2015
removed(tf) = 1; % apply local outlier

% Apply outlier removal to easting and northing velocity
removed(slow,:) = 0;
removed = logical(removed);
vx_clean(removed) = NaN;
vy_clean(removed) = NaN;
vxE_clean(removed) = NaN;
vyE_clean(removed) = NaN;

% Check for slow flowing ice again
vx_clean(slow,:) = repmat(ref.gate_vx(slow),[1,length(time_vel)]);
vy_clean(slow,:) = repmat(ref.gate_vy(slow),[1,length(time_vel)]);

% clear some space
clear sp_raw spE_raw


%% Fill gaps

% initialise
vx_filled = vx_clean;
vy_filled = vy_clean;
vxE_filled = vxE_clean;
vyE_filled = vyE_clean;

% sort time
time_vel = time_vel(:)';
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time_vel));
tmp = time_vel + r;
tmp = sort(tmp);

% Fill small time-series gaps
filled = zeros(size(vx_filled));
for i = 1:length(gate_x)
    if isfinite(gate_x(i)) && slow(i)==0
        % vx
        ts = vx_filled(i,:);
        if sum(isfinite(ts))>1
            ts = interp1gap(tmp,ts,tmp,62,'linear','extrap',NaN);
        end
        vx_filled(i,:) = ts;
        % vy
        ts = vy_filled(i,:);
        if sum(isfinite(ts))>1
            ts = interp1gap(tmp,ts,tmp,62,'linear','extrap',NaN);
        end
        vy_filled(i,:) = ts;
    end
end

% Fill small (<=300 m) spatial gaps
for i = 1:length(time_vel)
    % vx
    prof = vx_filled(:,i);
    if sum(isfinite(prof))>1
        prof = interp1gap(prof,3,'linear','extrap',NaN);
        vx_filled(:,i) = prof;
    end
    % vy
    prof = vy_filled(:,i);
    if sum(isfinite(prof))>1
        prof = interp1gap(prof,3,'linear','extrap',NaN);
        vy_filled(:,i) = prof;
    end
end

% Fill remaining time-series gaps
for i = 1:length(gate_x)
    if isfinite(gate_x(i)) && slow(i)==0
        % vx
        ts = vx_filled(i,:);
        if sum(isfinite(ts))>1
            ts = interp1gap(tmp,ts,tmp,'linear','extrap',NaN);
            ts = fillmissing(ts,'next');
            ts = fillmissing(ts,'previous');
        end
        vx_filled(i,:) = ts;
        % vy
        ts = vy_filled(i,:);
        if sum(isfinite(ts))>1
            ts = interp1gap(tmp,ts,tmp,'linear','extrap',NaN);
            ts = fillmissing(ts,'next');
            ts = fillmissing(ts,'previous');
        end
        vy_filled(i,:) = ts;
    end
end

% Any timeseries that had just 1 or no observations still need filling.
% Fill these using the reference velocity
disp([ num2str(sum(isnan(vx_filled(:)))) ' nans remaining in vx_filled before using reference velocity']);
disp([ num2str(sum(any(isnan(vx_filled),2)) / size(vx_filled,1) * 100) ' % of the flux gates pixels need to use the reference velocity somewhere'  ]);
filled_with_ref = any(isnan(vx_filled),2);
for i = 1:length(time_vel)
    % vx
    tmp_vx = vx_filled(:,i);
    tmp_vxE = vxE_filled(:,i);
    isn = isnan(tmp_vx);
    tmp_vx(isn) = ref.gate_vx(isn);
    tmp_vxE(isn) = ref.gate_vx_err(isn);
    vx_filled(:,i) = tmp_vx;
    vxE_filled(:,i) = tmp_vxE;
    % vy
    tmp_vy = vy_filled(:,i);
    tmp_vyE = vyE_filled(:,i);
    isn = isnan(tmp_vy);
    tmp_vy(isn) = ref.gate_vy(isn);
    tmp_vyE(isn) = ref.gate_vy_err(isn);
    vy_filled(:,i) = tmp_vy;
    vyE_filled(:,i) = tmp_vyE;
end

% Check for slow flowing ice again
vx_filled(slow,:) = repmat(ref.gate_vx(slow),[1,length(time_vel)]);
vy_filled(slow,:) = repmat(ref.gate_vy(slow),[1,length(time_vel)]);

%% Smooth the time-series with a 60-day moving window

% prep time
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time_vel));
tmp = time_vel + r;
tmp = sort(tmp);

% Set error to 10% of velocity where we have filled gaps
isn = isnan(vxE_filled) | isnan(vyE_filled);
vxE_filled(isn) = abs(vx_filled(isn)).*0.1;
vyE_filled(isn) = abs(vy_filled(isn)).*0.1;

% Check for unrealistically large or small errors
vxE_filled(vxE_filled>5e3 | vxE_filled<=0) = abs(vx_filled(vxE_filled>5e3 | vxE_filled<=0)).*0.1;
vyE_filled(vyE_filled>5e3 | vyE_filled<=0) = abs(vy_filled(vyE_filled>5e3 | vyE_filled<=0)).*0.1;

% Apply smoother
vx_filled = movmean(vx_filled,93,2,'omitnan','samplepoints',tmp);
vx_filled = movmean(vx_filled,125,2,'omitnan','samplepoints',tmp);
vy_filled = movmean(vy_filled,93,2,'omitnan','samplepoints',tmp);
vy_filled = movmean(vy_filled,125,2,'omitnan','samplepoints',tmp);
vxE_filled = movmean(vxE_filled,93,2,'omitnan','samplepoints',tmp);
vxE_filled = movmean(vxE_filled,125,2,'omitnan','samplepoints',tmp);
vyE_filled = movmean(vyE_filled,93,2,'omitnan','samplepoints',tmp);
vyE_filled = movmean(vyE_filled,125,2,'omitnan','samplepoints',tmp);

% Check for slow flowing ice again
vx_filled(slow,:) = repmat(ref.gate_vx(slow),[1,length(time_vel)]);
vy_filled(slow,:) = repmat(ref.gate_vy(slow),[1,length(time_vel)]);

% Check for unrealistically large or small errors
vxE_filled(vxE_filled>5e3 | vxE_filled<=0) = abs(vx_filled(vxE_filled>5e3 | vxE_filled<=0)).*0.1;
vyE_filled(vyE_filled>5e3 | vyE_filled<=0) = abs(vy_filled(vyE_filled>5e3 | vyE_filled<=0)).*0.1;

% Make sure there are gaps in velocity where there are gaps in the gate
vx_filled(gate_nan,:) = NaN;
vy_filled(gate_nan,:) = NaN;
vxE_filled(gate_nan,:) = NaN;
vyE_filled(gate_nan,:) = NaN;
filled(isfinite(vx_filled) & isnan(vx_clean)) = 1;

% Save
save(['../out/int/gate_data/gate_vel_filled_migrate_' num2str(migration_distance) 'm.mat'],'vx_filled','vy_filled','vxE_filled','vyE_filled','time_vel','source_vel','filled','alignments','filled_with_ref');

end


%% function: align_measures_with_itslive_fit
function [speed,I_vs_M] = align_measures_with_itslive_fit(speed,source,time,slow)

% Get indices
overlap = time>datenum(1996,4,1) & time<datenum(2018,12,31);
ind_M = overlap & source==1;
ind_I = overlap & source==4;
fit_time = time(ind_I);

% add negligible random noise to time to avoid duplicates
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time));
time = time + r;
time_M = time(ind_M);
time_I = time(ind_I);

% Interpolate to same times
I_vs_M = zeros([size(speed,1),1]);
for i = 1:size(speed,1)
    ts_I = speed(i,ind_I);
    ts_M = speed(i,ind_M);
    if slow(i)==0
        if sum(isfinite(ts_I))>5 && sum(isfinite(ts_M))>5
            isf = isfinite(ts_M);
            [brob, ~] = robustfit(time_M(isf), ts_M(isf));
            SLF_M = brob(1) + brob(2).*fit_time;
            isf = isfinite(ts_I);
            [brob, ~] = robustfit(time_I(isf), ts_I(isf));
            SLF_I = brob(1) + brob(2).*fit_time;
            I_vs_M(i) = nanmean(SLF_I(:) - SLF_M(:));
        end
    end
end
I_vs_M(isnan(I_vs_M)) = 0;
     
% align
speed(:,source==1 | source==5) = speed(:,source==1 | source==5) + I_vs_M; % modify MEaSUREs speed

end

%% function: align_measures_with_itslive_fit
function [speed,I_vs_M] = align_measures_with_itslive_fit2(speed,source,time,slow)

% Get indices
overlap = time>datenum(2014,1,1) & time<datenum(2018,12,31);
ind_M = overlap & source==1;
ind_I = overlap & source==4;
fit_time = time(ind_I);

% add negligible random noise to time to avoid duplicates
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time));
time = time + r;
time_M = time(ind_M);
time_I = time(ind_I);

% Interpolate to same times
I_vs_M = zeros([size(speed,1),1]);
for i = 1:size(speed,1)
    ts_I = speed(i,ind_I);
    ts_M = speed(i,ind_M);
    if slow(i)==0
        if sum(isfinite(ts_I))>2 && sum(isfinite(ts_M))>2
            isf = isfinite(ts_M);
            [brob, ~] = robustfit(time_M(isf), ts_M(isf));
            SLF_M = brob(1) + brob(2).*fit_time;
            isf = isfinite(ts_I);
            [brob, ~] = robustfit(time_I(isf), ts_I(isf));
            SLF_I = brob(1) + brob(2).*fit_time;
            I_vs_M(i) = nanmean(SLF_I(:) - SLF_M(:));
        end
    end
end
I_vs_M(isnan(I_vs_M)) = 0;
     
% align
speed(:,ind_M) = speed(:,ind_M) + I_vs_M; % modify MEaSUREs speed

end

%% Function: align_measures_with_itslive
function [speed,I_vs_M] = align_measures_with_itslive(speed,source,time)

% get the MEaSUREs index
M_idx = find(source==1 & time>datenum(1996,1,1));
M_time = time(M_idx);
% Get the ITSLIVE index
I_idx = find(source==4 & time>datenum(1996,1,1));
I_time = time(I_idx);
% Extract the datasets
I_vel = speed(:,I_idx);
tmp = speed(:,M_idx);
M_vel = zeros(size(I_vel)).*NaN;
for j = 1:numel(I_idx)
    [~,nearidx] = findNearest(M_time,I_time(j));
    M_vel(:,j) = tmp(:,nearidx);
end
clear tmp nearidx
% Get the difference between datasets
I_vs_M = nanmean((I_vel - M_vel),2);
I_vs_M(isnan(I_vs_M)) = 0;
%   - adjust the MEaSUREs (annual and multiyear) velocity
speed(:,source==1 | source==5) = speed(:,source==1 | source==5) + I_vs_M;

end

%% Function: align_multiyear_with_itslive
function [speed,I_vs_MY] = align_multiyear_with_itslive(speed,source,time)

% get the MEaSUREs index
M_idx = find(source==5 & time>datenum(1996,1,1));
M_time = time(M_idx);
% Get the ITSLIVE index
I_idx = find(source==4 & time>datenum(1996,1,1));
I_time = time(I_idx);
% Extract the datasets
M_vel = speed(:,M_idx);
I_vel = zeros(size(M_vel)).*NaN;
I_vs_MY = zeros(size(M_vel)).*NaN;
for j = 1:numel(M_idx)
    time_idx = find(I_time>=M_time(j)-(365.25*2) & I_time<M_time(j)+(365.25*2)); % construct a 4-year average S1 velocity centred on each MEaSUREs estimate
    time_idx = I_idx(time_idx);
    I_vel(:,j) = nanmean(speed(:,time_idx),2);
    I_vs_MY(:,j) = (nanmean(speed(:,time_idx),2)) - M_vel(:,j);
    I_vs_MY(isnan(I_vs_MY)) = 0;
    speed(:,M_idx(j)) = speed(:,M_idx(j)) + I_vs_MY(:,j);
end
% Get the difference between datasets
%I_vs_MY = nanmean((I_vel - M_vel),2);
%I_vs_MY(isnan(I_vs_MY)) = 0;
%   - adjust the MEaSUREs (annual and multiyear) velocity
%speed(:,source==5) = speed(:,source==5) + I_vs_MY;

end

%% Function: align_multiyear_with_measures
function [speed,M_vs_MY] = align_multiyear_with_measures(speed,source,time)

% get the MEaSUREs multiyear index
M_idx = find(source==5 & time>datenum(1996,1,1));
M_time = time(M_idx);
% Get the MEaSUREs annual index
I_idx = find(source==1 & time>datenum(1996,1,1));
I_time = time(I_idx);
% Extract the datasets
M_vel = speed(:,M_idx);
I_vel = zeros(size(M_vel)).*NaN;
M_vs_MY = zeros(size(M_vel)).*NaN;
for j = 1:numel(M_idx)
    time_idx = find(I_time>=M_time(j)-(365.25*2) & I_time<M_time(j)+(365.25*2)); % construct a 4-year average S1 velocity centred on each MEaSUREs estimate
    time_idx = I_idx(time_idx);
    I_vel(:,j) = nanmean(speed(:,time_idx),2);
    M_vs_MY(:,j) = (nanmean(speed(:,time_idx),2)) - I_vel(:,j);
    M_vs_MY(isnan(M_vs_MY)) = 0;
    speed(:,M_idx(j)) = speed(:,M_idx(j)) + M_vs_MY(:,j);
end
% Get the difference between datasets
%M_vs_MY = nanmean((I_vel - M_vel),2);
%M_vs_MY(isnan(M_vs_MY)) = 0;
%%   - adjust the MEaSUREs (annual and multiyear) velocity
%speed(:,source==5) = speed(:,source==5) + M_vs_MY;

end

%% Function: align_annual_with_enveo
function [speed,E_vs_ann] = align_annual_with_enveo(speed,source,time)

% Get the MEaSUREs index
ann_idx = find(source==1 | source==4 & time>datenum(1996,1,1));
ann_time = time(ann_idx);
%  Get the ENVEO index
E_idx = find(source==2 & time>=datenum(2017,1,1));
E_time = time(E_idx);
% Extract the datasets
ann_vel = speed(:,ann_idx);
E_vel = zeros(size(ann_vel)).*NaN;
for j = 1:numel(ann_idx)
    if (ann_time(j)-(365.25/2))>=min(E_time)
        time_idx = find(E_time>=ann_time(j)-(365.25/2) & E_time<ann_time(j)+(365.25/2)); % construct an annual average S1 velocity centred on each MEaSUREs estimate
        time_idx = E_idx(time_idx);
        E_vel(:,j) = nanmean(speed(:,time_idx),2);
    end
end
%   - Get the difference between datasets
E_vs_ann = nanmean((ann_vel - E_vel),2);
E_vs_ann(isnan(E_vs_ann)) = 0;
% adjust the annual velocities
speed(:,source==1 | source==4 | source==5) = speed(:,source==1 | source==4 | source==5) - E_vs_ann;

end

%% Function: align_joughin_with_annual
function [speed,J_vs_ann] = align_annual_with_joughin(speed,source,time)

% get the MEaSUREs/ITSLIVE index
M_idx = find(source==1 | source==4 & time>datenum(2005,1,1));
M_time = time(M_idx);
% Get the Joughin index
J_idx = find(source==9 & time>datenum(2005,1,1));
J_time = time(J_idx);
% Extract the datasets
J_vel = speed(:,J_idx);
tmp = speed(:,M_idx);
M_vel = zeros(size(J_vel)).*NaN;
for j = 1:numel(J_idx)
    [~,nearidx] = findNearest(M_time,J_time(j));
    M_vel(:,j) = tmp(:,nearidx);
end
clear tmp nearidx
%   - Get the difference between datasets
J_vs_ann = nanmean((J_vel - M_vel),2);
J_vs_ann(isnan(J_vs_ann)) = 0;
%   - adjust the Joughin velocity
speed(:,source==9) = speed(:,source==9) - J_vs_ann;

end

%% Function: align_Isim_with_annual
function [speed,Isim_vs_ann] = align_annual_with_Isim(speed,source,time)

% get the MEaSUREs/ITSLIVE index
M_idx = find(source==1 | source==4 & time>datenum(2005,1,1));
M_time = time(M_idx);
% Get the Joughin index
J_idx = find(source==8 & time>datenum(2005,1,1));
J_time = time(J_idx);
% Extract the datasets
J_vel = speed(:,J_idx);
tmp = speed(:,M_idx);
M_vel = zeros(size(J_vel)).*NaN;
for j = 1:numel(J_idx)
    [~,nearidx] = findNearest(M_time,J_time(j));
    M_vel(:,j) = tmp(:,nearidx);
end
clear tmp nearidx
%   - Get the difference between datasets
Isim_vs_ann = nanmean((J_vel - M_vel),2);
Isim_vs_ann(isnan(Isim_vs_ann)) = 0;
%   - adjust the Joughin velocity
speed(:,source==8) = speed(:,source==8) - Isim_vs_ann;

end

%% Function align_SID_with_enveo
function [speed,E_vs_SID] = align_SID_with_enveo(speed,source,time,slow,gate_x,gate_y)

% Get indices
overlap = time>datenum(2018,4,1) & time<datenum(2021,12,31);
ind_E = overlap & source==2;
ind_S1 = overlap & source==6;
fit_time = time(ind_S1);

% add negligible random noise to time to avoid duplicates
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time));
time = time + r;
time_E = time(ind_E);
time_S1 = time(ind_S1);

% Interpolate to same times
E_vs_SID = zeros([size(speed,1),1]);
for i = 1:size(speed,1)
    ts_S1 = speed(i,ind_S1);
    ts_E = speed(i,ind_E);
    if slow(i)==0
        if sum(isfinite(ts_E))>20 && sum(isfinite(ts_S1))>20
            isf = isfinite(ts_E);
            [brob, ~] = robustfit(time_E(isf), ts_E(isf));
            SLF_E = brob(1) + brob(2).*fit_time;
            isf = isfinite(ts_S1);
            [brob, ~] = robustfit(time_S1(isf), ts_S1(isf));
            SLF_S1 = brob(1) + brob(2).*fit_time;
            E_vs_SID(i) = nanmean(SLF_S1(:) - SLF_E(:));
        end
    end
end
E_vs_SID(isnan(E_vs_SID)) = 0;
     
% align
for i = 1:size(speed,1)
    if E_vs_SID(i)>0
        speed(i,source~=6) = speed(i,source~=6) + E_vs_SID(i); % increase ENVEO speed
    elseif E_vs_SID(i)<0
        E_vs_SID(i) = abs(E_vs_SID(i));
        speed(i,source==6) = speed(i,source==6) + E_vs_SID(i); % increase SID speed
    end
end

% Over Getz, enveo performs much better
load('../out/int/basins/all_basins.mat');
basin_idx = find(strcmp(basins.name,'Getz')==1);
in = find(inpoly2([gate_x(:),gate_y(:)],[basins.x{basin_idx},basins.y{basin_idx}])==1);

% Merge the ENVEO and S1 data
idx = find(source==6);
tmp_time = time;
tmp_time(source~=2) = 0; % make sure we only hit enveo layers
for i = 1:length(idx)
    if time(idx(i))<datenum(2021,12,31)
        [~,E_idx] = findNearest(tmp_time,time(idx(i)));
        %isn = isnan(speed(:,E_idx));
        %tmp0 = speed(:,idx(i));
        %speed(isn,idx(i)) = NaN; % don't fill gaps with ENVEO
        tmp_data = speed(:,E_idx); 
        tmp_data = nanmean(tmp_data,2);
        isn = isnan(tmp_data);
        tmp_data(isn) = speed(isn,idx(i));
        speed(in,idx(i)) = tmp_data(in); % replace SID with average of SID and nearest ENVEO layer
    end
end

end

%% Function: align_annual_with_SID
function [speed,SID_vs_ann] = align_SID_with_annual_fit(speed,source,time,slow)

% Get indices
overlap = time>datenum(2015,1,1) & time<datenum(2020,12,31);
ind_M = overlap & source==1 | source==4;
ind_S = overlap & source==6;
fit_time = time(ind_S);

% add negligible random noise to time to avoid duplicates
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time));
time = time + r;
time_M = time(ind_M);
time_S = time(ind_S);

% Interpolate to same times
SID_vs_ann = zeros([size(speed,1),1]);
for i = 1:size(speed,1)
    ts_S = speed(i,ind_S);
    ts_M = speed(i,ind_M);
    if slow(i)==0
        if sum(isfinite(ts_S))>5 && sum(isfinite(ts_M))>5
            isf = isfinite(ts_M);
            [brob, ~] = robustfit(time_M(isf), ts_M(isf));
            SLF_M = brob(1) + brob(2).*fit_time;
            isf = isfinite(ts_S);
            [brob, ~] = robustfit(time_S(isf), ts_S(isf));
            SLF_S = brob(1) + brob(2).*fit_time;
            SID_vs_ann(i) = nanmean(SLF_S(:) - SLF_M(:));
        end
    end
end
SID_vs_ann(isnan(SID_vs_ann)) = 0;
     
% align
speed(:,source~=6 & source~=2) = speed(:,source~=6 & source~=2) - SID_vs_ann; % modify annual speed

end


%% Function: align_annual_with_SID
function [speed,SID_vs_ann] = align_SID_with_annual(speed,source,time)

% Get the MEaSUREs index
ann_idx = find(source==1 | source==4 & time>datenum(1996,1,1));
ann_time = time(ann_idx);
%  Get the ENVEO index
SID_idx = find(source==6 & time>=datenum(2015,1,1));
SID_time = time(SID_idx);
% Extract the datasets
ann_vel = speed(:,ann_idx);
SID_vel = zeros(size(ann_vel)).*NaN;
for j = 1:numel(ann_idx)
    if (ann_time(j)-(365.25/2))>=min(SID_time)
        time_idx = find(SID_time>=ann_time(j)-(365.25/2) & SID_time<ann_time(j)+(365.25/2)); % construct an annual average S1 velocity centred on each MEaSUREs estimate
        time_idx = SID_idx(time_idx);
        SID_vel(:,j) = nanmean(speed(:,time_idx),2);
    end
end
%   - Get the difference between datasets
SID_vs_ann = nanmean((ann_vel - SID_vel),2);
SID_vs_ann(isnan(SID_vs_ann)) = 0;
% adjust the annual velocities
speed(:,source~=6) = speed(:,source~=6) - SID_vs_ann;

end






